var _ = require('underscore');
var fs = require('fs');

var mainTemplatePath = '';
var encoding = 'utf-8';

function hasValidMainTemplate( callback ) {
	
	fs.exists( mainTemplatePath, function( exists ) {
		return callback( exists );
	} );
	
}

module.exports.setMainTemplatePath = function( path ) {
	mainTemplatePath = path;
};

module.exports.setEncoding = function( e ) {
	encoding = e;
};

var  async = require('async');

module.exports.setupApp = function( app, views_path ) {
	
	app.engine( 'use', function( filePath, options, callback ) {
		
		async.series([
			function( callback ) {
				
				if ( options.page === undefined ) options.page = {};
				if ( options.page.include === undefined ) options.page.include = {};
				if ( options._includes === undefined ) options._includes = {};
				options.page.path = filePath;
				
				callback();
				
			},
			function( callback ) {
				
				for ( var includeid in options._includes ) {
					
					fs.readFile( options._includes[ includeid ], encoding, function( err, includeFileContent ) {
						
						var compiledInclude = _.template( includeFileContent );
						options.page.include[ includeid ] = compiledInclude( options );
						
					} );
					
				}
				
				callback();
				
			},
			function( callback ) {
				
				fs.readFile( filePath, encoding, function( err, pageData ) {
					
					if ( err ) return callback( err );
					
					var compiledPage = _.template( pageData );
					options.page.content = compiledPage( options );
					
					callback();
					
				}
				
			},
			function( callback ) {
				
				hasValidMainTemplate( function( exists ) {
					
					if ( exists ) {
						
						fs.readFile( mainTemplatePath, encoding, function( err, templateData ) {
								
							if ( err ) return callback( err );
							
							var compiledTemplate = _.template( templateData );
							var rendered = compiledTemplate( options );
							
							return callback( null, rendered );
							
						} );
						
					} else {
						
						var compiled = _.template( pageData );
						var rendered = compiled( options );
						
						return callback( null, rendered );
						
					}
					
				} );
				
			}
		], function(err, res) {
			if (err !== undefined || res == 1) {
				callback(err, res);
			}
		});
		
	} );
	
	app.set( 'views', views_path );
	app.set( 'view engine', 'use' );
	
}